/**
 * @file
 * Snippet manager behaviors.
 */

(function ($, Drupal) {

  'use strict';

  Drupal.behaviors.snippetManager = {
    attach: function (context, settings) {

      var $textArea = $('textarea[data-snippet-code]')
        .once('snippet_manager_editor');

      if (!$textArea[0]) {
        return;
      }

      /* global CodeMirror */
      if (typeof CodeMirror === 'undefined') {
        alert(Drupal.t('CodeMirror library is not loaded!'));
        return;
      }

      function getModes() {
        var modes = {};
        var modesEncoded = $.cookie('snippetModes');
        if (modesEncoded) {
          modes = JSON.parse(modesEncoded);
        }
        return modes;
      }

      function setFullScreen(state) {
        editor.setOption('fullScreen', state);
        $('svg[data-sm-button="enlarge"]').toggle(!state);
        $('svg[data-sm-button="shrink"]').toggle(state);
      }

      function createList(type) {
        var list = '<' + type + '>\n';
        doc.getSelection().split('\n').forEach(function (value) {
          list += '  <li>' + value + '</li>\n';
        });
        list += '</' + type + '>\n';
        doc.replaceSelection(list, doc.getCursor());
      }

      // Bubble error class.
      if ($textArea.hasClass('error')) {
        $textArea.parent().addClass('snippet-error');
      }

      // Remove "required" attribute because the textarea is not focusable.
      $textArea.removeAttr('required');

      // Create HTML/Twig overlay mode.
      CodeMirror.defineMode('html_twig', function (config, parserConfig) {
        return CodeMirror.overlayMode(
          CodeMirror.getMode(config, parserConfig.backdrop || 'text/html'),
          CodeMirror.getMode(config, 'twig')
        );
      });

      var mode = getModes()[settings.path.currentPath] || $textArea.data('mode');

      var editor = CodeMirror.fromTextArea($textArea[0], {
        lineNumbers: true,
        mode: mode,
        theme: settings.snippetManager.editor.codeMirror.theme,
        extraKeys: {
          F11: function (editor) {
            setFullScreen(!editor.getOption('fullScreen'));
          },
          Esc: function (editor) {
            setFullScreen(false);
          }
        }
      });

      var doc = editor.getDoc();

      // Insert variables into editor.
      $('[data-drupal-selector="snippet-variable"]').click(function () {
        doc.replaceSelection('{{ ' + $(this).text() + ' }}', doc.getCursor());
        return false;
      });

      // Toolbar.
      var $editor = $('.CodeMirror');

      $('<div id="snippet-manager-buttons"/>')
        .prependTo($editor)
        .load(settings.snippetManager.editor.buttonsBaseUrl);

      var $toolbar = $('<div class="snippet-manager-toolbar"/>')
        .prependTo($editor);

      var buttons = [
        'bold',
        'italic',
        'underline',
        'strike-through',
        'list-numbered',
        'list-bullet',
        'horizontal-rule',
        'undo',
        'redo',
        'clear-formatting',
        'enlarge',
        'shrink'
      ];

      buttons.forEach(function (button, title) {
        if (typeof $textArea.attr('data-btn-' + button) !== 'undefined') {
          // @TODO: Add title attribute.
          $('<svg data-sm-button="' + button +'" class="sm-button"><use xlink:href="#icon-' + button + '"></use></svg>')
            .appendTo($toolbar);
          $('[data-sm-button="shrink"]').hide();
        }
      });

      if (typeof $textArea.attr('data-mode-select') !== 'undefined') {
        $('<select class="sm-mode"/>')
          .append('<option value="html_twig">HTML/Twig</option>')
          .append('<option value="text/html">HTML</option>')
          .append('<option value="twig">Twig</option>')
          .append('<option value="javascript">JavaScript</option>')
          .append('<option value="css">CSS</option>')
          .val(mode)
          .change(function () {
            var value = $(this).val();
            editor.setOption('mode', value);
            // Save the value to cookie.
            var modes = getModes();
            modes[settings.path.currentPath] = value;
            $.cookie('snippetModes', JSON.stringify(modes), {path: settings.path.baseUrl});

          })
          .appendTo($toolbar);
      }

      $('svg[data-sm-button]').click(function (event) {
        var button = $(event.target).closest('[data-sm-button]').data('sm-button');
        switch (button) {

          case 'bold':
            doc.replaceSelection('<strong>' + doc.getSelection() + '</strong>', doc.getCursor());
            break;

          case 'italic':
            doc.replaceSelection('<em>' + doc.getSelection() + '</em>', doc.getCursor());
            break;

          case 'underline':
            doc.replaceSelection('<u>' + doc.getSelection() + '</u>', doc.getCursor());
            break;

          case 'strike-through':
            doc.replaceSelection('<s>' + doc.getSelection() + '</s>', doc.getCursor());
            break;

          case 'list-numbered':
            createList('ol');
            break;

          case 'list-bullet':
            createList('ul');
            break;

          case 'horizontal-rule':
            doc.replaceSelection('<hr/>', doc.getCursor());
            break;

          case 'undo':
            doc.undo();
            break;

          case 'redo':
            doc.redo();
            break;

          case 'clear-formatting':
            doc.replaceSelection($('<div>' + doc.getSelection() + '</div>').text(), doc.getCursor());
            break;

          case 'enlarge':
            setFullScreen(true);
            break;

          case 'shrink':
            setFullScreen(false);
            break;

        }

      });

    }
  };

}(jQuery, Drupal));
