<?php

namespace Drupal\snippet_manager\Commands;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Asset\AssetCollectionOptimizerInterface;
use Drupal\Core\Asset\LibraryDiscoveryInterface;
use Drupal\Core\State\StateInterface;
use Drush\Commands\DrushCommands;
use GuzzleHttp\Client;
use Symfony\Component\HttpFoundation\Response;

/**
 * Drush integration for Snippet manager module.
 */
class SnippetManagerCommands extends DrushCommands {

  /**
   * Relative path to CodeMirror directory.
   */
  const LIBRARY_PATH = 'libraries/codemirror';

  /**
   * Base CDN URL to CodeMirror library.
   */
  const CDN_BASE_URL = 'https://cdnjs.cloudflare.com/ajax/libs/codemirror/';

  /**
   * Library discovery service.
   *
   * @var \Drupal\Core\Asset\LibraryDiscoveryInterface
   */
  protected $libraryDiscovery;

  /**
   * The HTTP client.
   *
   * @var \GuzzleHttp\Client
   */
  protected $httpClient;

  /**
   * JS collection optimizer.
   *
   * @var \Drupal\Core\Asset\AssetCollectionOptimizerInterface
   */
  protected $jsCollectionOptimizer;

  /**
   * CSS collection optimizer.
   *
   * @var \Drupal\Core\Asset\AssetCollectionOptimizerInterface
   */
  protected $cssCollectionOptimizer;

  /**
   * The state key value store.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected $state;

  /**
   * The time service.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected $time;

  /**
   * Constructs the object.
   *
   * @param \Drupal\Core\Asset\LibraryDiscoveryInterface $library_discovery
   *   The library discovery service.
   * @param \GuzzleHttp\Client $http_client
   *   The HTTP client.
   * @param \Drupal\Core\Asset\AssetCollectionOptimizerInterface $js_collection_optimizer
   *   The JS asset collection optimizer service.
   * @param \Drupal\Core\Asset\AssetCollectionOptimizerInterface $css_collection_optimizer
   *   The CSS asset collection optimizer service.
   * @param \Drupal\Core\State\StateInterface $state
   *   The state key value store.
   * @param \Drupal\Component\Datetime\TimeInterface $time
   *   The time service.
   */
  public function __construct(
    LibraryDiscoveryInterface $library_discovery,
    Client $http_client,
    AssetCollectionOptimizerInterface $js_collection_optimizer,
    AssetCollectionOptimizerInterface $css_collection_optimizer,
    StateInterface $state,
    TimeInterface $time
  ) {
    parent::__construct();
    $this->libraryDiscovery = $library_discovery;
    $this->httpClient = $http_client;
    $this->jsCollectionOptimizer = $js_collection_optimizer;
    $this->cssCollectionOptimizer = $css_collection_optimizer;
    $this->state = $state;
    $this->time = $time;
  }

  /**
   * Downloads CodeMirror library.
   *
   * @command sm:download-codemirror
   */
  public function downloadCodemirror() {
    $io = $this->io();

    $cm_library = $this->libraryDiscovery->getLibraryByName('snippet_manager', 'codemirror');

    $source_base_path = self::CDN_BASE_URL . $cm_library['version'];
    $destination_base_path = DRUPAL_ROOT . '/' . self::LIBRARY_PATH;

    $assets = array_merge($cm_library['js'], $cm_library['css']);
    foreach ($assets as $asset) {

      if (empty($asset['type']) || $asset['type'] != 'external') {
        $source = $source_base_path . explode(self::LIBRARY_PATH, $asset['data'])[1];
        $destination = DRUPAL_ROOT . '/' . $asset['data'];
      }
      else {
        $source = $asset['data'];
        $destination = $destination_base_path . explode($source_base_path, $asset['data'])[1];
      }

      $io->write("<info>$source</info>");

      $directory = dirname($destination);
      if (!file_prepare_directory($directory, FILE_CREATE_DIRECTORY)) {
        $io->error("Could not create directory $directory.");
        return 1;
      }

      try {
        $response = $this->httpClient->get($source, ['sink' => $destination]);
      }
      catch (\Exception $exception) {
        $io->error($exception->getMessage());
        return 1;
      }

      $status_code = $response->getStatusCode();
      Response::$statusTexts[$status_code];
      $io->writeln(' [' . $status_code . ' ' . Response::$statusTexts[$status_code] . ']');
    }

    // Clear asset caches.
    $this->jsCollectionOptimizer->deleteAll();
    $this->cssCollectionOptimizer->deleteAll();
    $this->state->set('system.css_js_query_string', base_convert($this->time->getRequestTime(), 10, 36));

    $io->success("CodeMirror library has been downloaded into $destination_base_path directory.");
  }

}
